"use strict";

const aFrom          = require("es5-ext/array/from")
    , objForEach     = require("es5-ext/object/for-each")
    , isObject       = require("es5-ext/object/is-object")
    , ensureFunction = require("es5-ext/object/ensure-plain-function")
    , ensureString   = require("es5-ext/object/validate-stringifiable-value");

module.exports = function (moduleIds, callback) {
	// 1. Validate & resolve input
	if (isObject(moduleIds)) {
		moduleIds = aFrom(moduleIds);
		if (!moduleIds) throw new TypeError("Minimum one moduleId is required");
	} else {
		moduleIds = [ensureString(moduleIds)];
	}
	ensureFunction(callback);

	// 2. Cache currently cached module values
	const cache = {};
	moduleIds.forEach(moduleId => {
		cache[moduleId] = require.cache[moduleId];
		delete require.cache[moduleId];
	});

	try {
		// 3. Run callback
		return callback();
	} finally {
		// 4. Restore state
		objForEach(cache, (value, moduleId) => {
			if (value) require.cache[moduleId] = value;
			else delete require.cache[moduleId];
		});
	}
};
